/* 
-----------------------------------------------------------------------------
File: dba.js
Author: Karl Bishop (kfbishop@us.ibm.com)
-----------------------------------------------------------------------------
This is a custom Dojo module used in DojoDBA application
-----------------------------------------------------------------------------
(C) Copyright IBM Corp. 2008 All rights reserved.

This sample program is owned by International Business Machines Corporation or
one of its subsidiaries ("IBM") and is copyrighted and licensed, not sold.

The sample program is provided to you on an "AS IS" basis, without warranty of
any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER
EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE. Some jurisdictions do not allow for the exclusion or limitation of
implied warranties, so the above limitations or exclusions may not apply to you.
IBM shall not be liable for any damages you suffer as a result of using, modifying
or distributing the sample program or its derivatives.

Each copy of any portion of this sample program or any derivative work, must
include the above copyright notice and disclaimer of warranty.
----------------------------------------------------------------------------
*/
if(!dojo._hasResource["dba"]) {
dojo._hasResource["dba"] = true;
dojo.provide("dba");

dojo.require("dojox.widget.Toaster");
dojo.require("dojox.string.Builder");

//-- Remote REST URL's used by the application.  
//-- Parameters are replaced at runtime.
dba.urls = {
	DS:     "/resources/ds",
	SCHEMA: "/resources/ds/${ds}?showSysTables=${showSysTables}",
	DBINFO:	"/resources/dbInfo/${ds}",
	SQL:    "/resources/ds/${ds}/sql",
	TABLE:  "/resources/ds/${ds}/tables/${table}"
};

//-- All of our data stores used within the application
dba.stores = {
	dataSources: null,
	schema: null,
	tableDetails: null
};

dba.dataSource = null;
dba.busyMessage = [];

//------------------------------------------------------------------------
dba.init = function() {
	var F = "dba.init(): ";
	console.debug(F, "Starting...");
	
	//-- Create a common toaster notifier if not exists
	var div = document.createElement("div");
    div.setAttribute("id", "messageToaster");
    document.body.appendChild(div);
    var toaster = new dojox.widget.Toaster( {
    	separator: "<hr/>",
        positionDirection: "br-up",  /* bottom-right, move up */
        duration: 2000,     /* 2 secs */
        messageTopic: "toasterMessageTopic"
    }, "messageToaster" );
	dba.loadDataSources();
};

//------------------------------------------------------------------------
dba.loadDataSources = function() {
	// summary:
	//	Load iniital Data Source list
	var F = "dba.loadDataSources(): ";
	console.debug(F, "Starting...");
	dba.busy("Data Sources", true);
	dba.stores.dataSource = new dojo.data.ItemFileReadStore( {url: dba.urls.DS} );
	dba.stores.dataSource.fetch({
		sort: [{attribute:"uid"}],
		start:0,
		count:1,
		onComplete: function(items, request) {
			try {
				console.debug(F, "Data Sources loaded.");
				var key = dba.stores.dataSource.getValue(items[0], "uid");
				console.debug( F, "Setting DS default to: ", key );

				var dsInput = dijit.byId("dataSource");
				dsInput.store = dba.stores.dataSource;
				dsInput.setValue( key );
				dsInput.setDisabled( false );
				
				var dsButton = dijit.byId("loadDataSourceButton");
				dsButton.onClick = dba.loadDatabaseSchema;
				dsButton.setDisabled( false );

				dba.busy("Data Sources", false);
			} catch ( e  ) {
				console.error( F, e );
				dba.busy();
			}
		}
	});
};

//------------------------------------------------------------------------
dba.loadDatabaseSchema = function() {
	// summary:
	//		Load Schema Data Store for desired Data Source
	var F = "dba.loadDatabaseSchema(): ";

	dba.dataSource = dijit.byId("dataSource").getValue();
	console.debug(F, "Loading ", dba.dataSource);

	//-- Update title
	dojo.byId("dataSourceTitle").innerHTML = dba.dataSource;

	//-- Load up dbInfo
	dba.loadDatabaseInfo();

	//-- Load up DataStore
	dba.busy("Database Schema", true);
	var showSys = dijit.byId("showSysTables").checked;
	console.debug("ShowSysTable = ", dijit.byId("showSysTables"));
	var url = dojo.string.substitute( dba.urls.SCHEMA, {ds:dba.dataSource, showSysTables: showSys} );
	dba.stores.schema = new dojo.data.ItemFileReadStore( {url:url} );
	dba.stores.schema.fetch({onComplete: dba.schemaLoaded});
	
};
	
//------------------------------------------------------------------------
dba.loadDatabaseInfo = function() {
	var F = "dba.loadDatabaseInfo(): ";
	dba.dataSource = dijit.byId("dataSource").getValue();
	console.debug(F, "Loading ", dba.dataSource);

	//-- Load up dbInfo too
	dba.busy("Database Info", true);
	dojo.byId("dbInfo_dbUrl").innerHTML = "Loading...";
	dojo.byId("dbInfo_dbName").innerHTML = "Loading...";
	dojo.byId("dbInfo_dbVersion").innerHTML = "Loading...";
	dojo.byId("dbInfo_driverName").innerHTML = "Loading...";
	dojo.xhrGet( {
		url: dojo.string.substitute( dba.urls.DBINFO, {ds:dba.dataSource} ),
		handleAs: "json",
		load: function(response, ioArgs) {
			console.debug(F, "Received DBInfo: ", response);
			dojo.byId("dbInfo_dbUrl").innerHTML = response.db_url;
			dojo.byId("dbInfo_dbName").innerHTML = response.db_name;
			dojo.byId("dbInfo_dbVersion").innerHTML = response.db_version;
			dojo.byId("dbInfo_driverName").innerHTML = response.driver_name;
//			dojo.byId("dbInfo_driverVersion").innerHTML = response.driver_version;
			dba.busy("Database Info", false);
		},
		error: function(response, ioArgs) {
			console.error(F,"load error: ", response, ioArgs);
			dba.busy("Database Info", false);
		}
	} );
};
	
//------------------------------------------------------------------------
dba.schemaLoaded = function() {
	var F = "dba.schemaLoaded(): ";
	try {
		console.debug(F, "Schema store loaded: ", dba.stores.schema);
		//-- Create Schema Tree
		dba.createSchemaTree();
		//-- Create and Enable our Editor
		dba.createSqlEditor();
	} catch(e) {
		console.error(F,e);
	}
	dba.busy("Database Schema", false);
};

//------------------------------------------------------------------------
dba.createSchemaTree = function() {
	var F = "dba.createSchemaTree(): ";
	try {
		console.debug(F, "Starting");
		var schemaTree = dijit.byId("schemaTree");
		if ( schemaTree ) {
			console.debug(F, "Destroying existing schema tree");
			schemaTree.destroyRecursive();
			schemaTree.destroy();
		}
		var div = dojo.byId("schemaTreeContainer");
		div.innerHTML = "";
		div.appendChild( document.createElement("div") );
		schemaTree = new dijit.Tree({
  			id: 'schemaTree',
			store: dba.stores.schema,
    	  	labelAttr: 'label',
     		typeAttr: "type",
	    	onClick: dba.tableSelected
		});
		div.appendChild( schemaTree.domNode );
	
		//-- Advance accordion to schema tree pane	
		dijit.byId("accordion").forward();
	} catch(e) {console.error(F,e);}
};

//------------------------------------------------------------------------
dba.createSqlEditor = function() {
	var F = "dba.createSqlEditor(): ";
	try {
		console.debug(F, "Starting");
		var sqlEditor = dijit.byId("sqlEditor");
		if ( ! sqlEditor ) {
			sqlEditor = new dijit.Editor({
	        	plugins: ['undo','redo','|','copy','cut','paste','|','indent','outdent','|']
			}, dojo.byId("sqlEditor") );
		}
		sqlEditor.setValue("Enter SQL here, or select a table from the Schema tree to generate a sample query");
		
		//-- Set up our execute button
		dijit.byId("executeButton").setDisabled(false);
		
	} catch(e) {console.error(F,e);}
};

//------------------------------------------------------------------------
dba.executeSql = function() {
	var F = "dba.executeSql(): ";
	console.debug(F, "Starting...");
	dba.busy("SQL Query", true);

	try {
		var sql = dijit.byId("sqlEditor").getValue();
		//-- Editor uses a lot of HTML for formatting. We need to strip them for SQL string
		console.debug(F,"SQL = ", sql);
  		sql = sql.replace(/<[^<>]*>/g, " ");
  		console.debug(F,"SQL = ", sql);
		dojo.xhrPost( {
			url: dojo.string.substitute(dba.urls.SQL, {ds:dba.dataSource}),
			content: {
				"sql": sql
			},
			handleAs: "json",
			timeout: 60000,
			load: function(response, ioArgs) {
				dba.processSqlResults(sql, response);
			}
		} );
	} catch(e) {
		console.error(F,e);
		dba.busy("SQL Query", false);
	}
};

//------------------------------------------------------------------------
dba.processSqlResults = function( sql, response ) {
	var F = "dba.processSqlResults(): ";
	console.debug(F, "Starting...");

	try {
		console.debug(F,"SQL: ", sql);
		console.debug(F,"Response: ", response);
		var div;
		if ( "Error" == response.messageType ) {
			console.error(F,"Response message: ", response.message);
			div = document.createElement("div");
			div.className = 'error';
			div.innerHTML = response.message;
			dojo.byId("resultsContainer").appendChild( div );
		} else if ( "Info" == response.messageType ) {
			console.info(F,"Response message: ", response.message);
			if( response.content && response.content.items.length > 0 ) {
				// Have a table, so show a toaster message
			    dojo.publish("toasterMessageTopic", [{
			    	message: response.message
			     }] );
			} else {
				// no table, so just show the message
				div = document.createElement("div");
				div.className = 'title';
				div.innerHTML = response.message;
				dojo.byId("resultsContainer").appendChild( div );
			}
		}

		//-- Build a grid if headers and contents are here
		if ( response.headers && response.content ) {
			//-- Build our layout form the headers
			var resultsGridLayout = null;
			var cells = [];
			dojo.forEach( response.headers, function(hdr) {
				var cell = {name:hdr.name, field:hdr.key};
    	    	if ( "number" == hdr.type) {
					cell.cellStyles = "text-align: right;";
				} else if ( "boolean" == hdr.type ) {
					cell.cellStyles = "text-align: center";
				}
				cells[cells.length] = cell;
			});
			resultsGridLayout = [{cells:[ cells ]}];
			console.debug(F,"ResultsGridLayout = ", resultsGridLayout);

			//-- Create a grid for the response
			var resultsContentStore = new dojo.data.ItemFileReadStore( {data:response.content} );
			var resultsGridModel = new dojox.grid.data.DojoData(null, resultsContentStore, {
				query: {C1:'*'},
				clientSort: true,
				rowsPerPage: 20
			});
			console.debug(F,"ResultsGridModel: ", resultsGridModel);
						
			var resultsGrid = dijit.byId("resultsGrid");
			if ( ! resultsGrid ) {
				resultsGrid = new dojox.Grid({
					id: "resultsGrid",
					structure: resultsGridLayout,
				    model: resultsGridModel,
				    autowidth: true,
			    	autoheight: true
				}, dojo.byId("resultsContainer") );
			} else {
				resultsGrid.setModel( resultsGridModel );
				resultsGrid.setStructure( resultsGridLayout );
			}
			setTimeout(function() {	resultsGrid.update(); }, 300);

		} //if headers & content
		dba.busy("SQL Query", false);

	} catch(e) {console.error(F,e);}
	dba.busy("SQL Query", false);
};

//------------------------------------------------------------------------
dba.tableSelected = function( item ) {
	var F = "dba.tableSelected(): ";
	try {
		var table = dba.stores.schema.getValue(item,"uid");
		console.log(F, "Table ", table, item);
		var type = dba.stores.schema.getValue(item, "type"); 
		if ( type == "table" ) {
			console.log(F, "Got a table, building Sample SQL and results table");
			dba.busy("Table Details", true);
			var url = dojo.string.substitute( dba.urls.TABLE, {ds:dba.dataSource, table:table} );
			dba.stores.tableDetails = new dojo.data.ItemFileReadStore( {url:url} );
			dba.stores.tableDetails.fetch({
				onComplete: function(items, request) {
					try {
						console.debug(F, "Table details loaded for: ", table, ", data: ", dba.stores.tableDetails);
						
                        //-- Populate editor with sample SQL for this table
                        //-- Use full editor based HTML formatting, just because.
                        //-- select<br /><blockquote>firstname,<br />lastname<br /></blockquote>from<br /><blockquote>DB2INST1.EMPLOYEE</blockquote>
                        console.debug(F,"Building editor query string for table: ", table);
                        var sql = new dojox.string.Builder();
                        sql.append("<b>SELECT</b><br /><blockquote>");
                        var cntr = 0;
                        dojo.forEach(items, function(i) {
                        	if ( cntr++ > 0 ) {
                        		sql.append(", ");
                        	}
       						sql.append( dba.stores.tableDetails.getValue(i, "name") );
       					} );
       					sql.append("<br /></blockquote>");
       					sql.append("<b>FROM</b><br /><blockquote>");
       					sql.append( table );
       					sql.append("</blockquote><br />");
						dijit.byId("sqlEditor").setValue( sql.toString() );
                        
						//-- Put table name in the pane
						dojo.byId('tableDetailsName').innerHTML = table;
						
						//-- Advance accordion to schema tree pane	
						dijit.byId("accordion").forward();

						//-- Create a grid for the table
						var tableDetailsGridModel = new dojox.grid.data.DojoData(null, dba.stores.tableDetails, {
							query: {name:'*'},
							clientSort: true,
							rowsPerPage: 20
						});
						
						//-- Assign default widths for the columns
						var tableDetailsGridLayout = [{cells:[[
							{name:'Column Name', field:'name'},
							{name:'Column Type', field:'type'}
						]]}];
						
						var tableDetailsGrid = dijit.byId("tableDetailsGrid");
						
						if ( ! tableDetailsGrid ) {
							tableDetailsGrid = new dojox.Grid({
								id: "tableDetailsGrid",
    							structure: tableDetailsGridLayout,
							    model: tableDetailsGridModel,
							    autowidth: true,
							    autoheight: true
							}, dojo.byId("tableDetailsContainer") );
						} else {
							tableDetailsGrid.setModel( tableDetailsGridModel );
						}
						setTimeout(function() {
							tableDetailsGrid.update();
                        }, 1);
                        
						dba.busy("Table Details", false);
					} catch ( e  ) {
						console.error( F, e );
						dba.busy("Table Details", false);
					}
				}
			});
		} else {
			console.log(F, "Not a table (", type, ")");
		    dojo.publish("toasterMessageTopic", [{
		    	message: "Select a table type object to view details"
		     }] );
		}
	} catch(e) {console.error(F,e);}
};

//------------------------------------------------------------------------
dba.busy = function( /*string*/topic, /*bool*/start ) {
	var F = "dba.busy(): ";
	console.debug(F, "Topic=",topic, ", Start=", start );
	
	if( start ) {
		// Add item to list
		dba.busyMessage.push(topic);
	} else {
		// Remove from list
		dba.busyMessage.splice( dba.busyMessage.indexOf(topic), 1 );
	}
	
	if ( dba.busyMessage.length > 0 ) {
		dojo.byId('busyBar').style.visibility='visible';
    	dojo.fadeIn({ node: 'busyBar', duration:300 }).play();
    	dijit.byId('busy').update({ maximum: 1, progress: 1 } );
   	} else {
       	dojo.fadeOut({ node: 'busyBar', duration:800,
	       	onEnd: function(){
            	// remove progress bar from tab order
               	dojo.byId('busyBar').style.visibility='hidden';
       		}
		}).play();
   	}
};
	
//------------------------------------------------------------------------
dba.busyReport = function( percent ) {
   	return "Processing: " + dba.busyMessage.toString();
};

//------------------------------------------------------------------------
dba.toggleTheme = function() {
   	document.body.className = (document.body.className == "soria" ? "tundra" : "soria");
   	console.info("Theme set to: ", document.body.className);
};

}